module WorkTimer

class SimpleReportFormatter

public
   def format(timeClock, startTime, endTime)
      retString = ""
      reportHourSummary = 0
      
      eventHash = timeClock.createEventHash(startTime, endTime)
      
      #Divide the time interval into days
      numDays = (endTime - startTime) / (3600 * 24)
      
      if (endTime - startTime) % (3600 * 24) > 0 then
         numDays += 1
      end
      
      startTimeString = Time.at(startTime).strftime("%m/%d/%Y")
      
      # note: we're actually producing a report for the interval
      # [startTime, endTime) so we should subtract a second from
      # the end time string so that we don't lead the user to
      # think that the report includes the next day
      endTimeString = Time.at(endTime-1).strftime("%m/%d/%Y")
      
      retString += "---------------------------------------------------------\n"
      retString += "| Activity Report (#{startTimeString} - #{endTimeString})\n"
      retString += "---------------------------------------------------------\n"
      #get a summary of activities for each day
      1.upto(numDays) {
         |dayCount|
         dayStart = startTime + (dayCount - 1) * 3600 * 24
         dayEnd = dayStart + 3600 * 24
           
         timeString = Time.at(dayStart).strftime("%a %b %d, %Y")
         
         itemIndex = 1
         
         retString += "| #{timeString}:\n"
         
         dailyHourSummary = 0
         
         eventHash.each_pair {
            |description, eventContainer|
            
            #get the amount of time spent in hours
            totalEventTime = eventContainer.computeTimeSpent(dayStart, dayEnd)
            
            totalHrs      = totalEventTime / 3600.0
            #remainMinutes = (totalEventTime % 3600) / 60
            
            dailyHourSummary += totalHrs
            
            if totalEventTime > 0 then
               retString += sprintf("|    #{itemIndex}. %-20s: %.2f hrs\n", description, totalHrs)
               itemIndex += 1
            end
         }  
         
         if dailyHourSummary > 0 then
            retString += sprintf("|    --->Total: %.2f hrs\n", dailyHourSummary)
            reportHourSummary += dailyHourSummary
         end      
      }
      
      retString += "---------------------------------------------------------\n"
      retString += sprintf("| Total: %.2f hrs\n", reportHourSummary)
      retString += "---------------------------------------------------------\n"

      return retString
   end
end

end