require_gem 'fxruby'
require 'time'
require 'ApplicationProperties'

include Fox

module WorkTimer

class EventTableItem < FXTableItem
  attr_accessor :theEvent
  
  def initialize(text, event)
    super(text, nil, nil)
    @theEvent = event
  end
end

class TimeEditorDialog < FXDialogBox
  attr_accessor :eventsTable
  attr_reader :theEventList
  attr_reader :eventsToDelete
  attr_reader :applyButton

   def doWindowPropsChanged(source, selector, event)
      timeEditorProps = ApplicationProperties.instance.propertyHash["TimeEditorDialogInfo"]
      if self.width != timeEditorProps["width"].to_i or
         self.height != timeEditorProps["height"].to_i then
         timeEditorProps["width"] = self.width.to_s
         timeEditorProps["height"] = self.height.to_s
      end
          
   end
  
  def initialize(owner, eventList)
    # Invoke base class initialize function first
    super(owner, "View/Edit Event Data", DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE, 0, 0, 305, 400)

      # handle window resizes and moves
      connect(SEL_CONFIGURE, method(:doWindowPropsChanged))
      
      timeEditorProps = ApplicationProperties.instance.propertyHash["TimeEditorDialogInfo"]
      
      self.resize(
         timeEditorProps["width"].to_i,
         timeEditorProps["height"].to_i)
      
    @theEventList = eventList
    
    @eventsToDelete = Array.new
    
   windowContent = 
      FXVerticalFrame.new(
         self, 
         LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,
         0, 0, 0, 0, 0, 0, 0, 0)

    eventsTableGroup = 
        FXGroupBox.new(
           windowContent, 
           "Event Data",
           GROUPBOX_TITLE_LEFT|FRAME_RIDGE|LAYOUT_FILL_X|LAYOUT_FILL_Y)

    # let's add the table
    @eventsTable = FXTable.new(eventsTableGroup, nil, 0, TABLE_NO_COLSELECT|TABLE_COL_SIZABLE|LAYOUT_FILL_X|LAYOUT_FILL_Y)
    @eventsTable.appendColumns(3)
    @eventsTable.setColumnText(0, "Event Type")
    @eventsTable.setColumnWidth(0, 70)
    @eventsTable.setColumnText(1, "Activity Name") 
    @eventsTable.setColumnText(2, "Event Time")    
    @eventsTable.setColumnWidth(2, 120)
    @eventsTable.connect(SEL_DESELECTED, method(:cellDeselected))
    
    populateTable

    # if the minimum length of an activity in the table
    # is less than 80, let's manually set the size of
    # the activites column to 80 so that the column
    # header is fully displayed
    @eventsTable.fitColumnsToContents(1)
    if @eventsTable.getColumnWidth(1) < 80 then
        @eventsTable.setColumnWidth(1, 80)
    end
     
    @eventsTable.rowHeaderWidth = 0
    
    #add a right-click handler for the events table
    @eventsTable.connect(SEL_RIGHTBUTTONRELEASE, method(:showTableContextMenu))
               
    #Close
    buttons = FXHorizontalFrame.new(windowContent, LAYOUT_RIGHT)

    @applyButton = FXButton.new(
        buttons, 
        "&Apply Changes", 
        nil, 
        nil, 
        0)

    @applyButton.connect(SEL_COMMAND, method(:applyChanges))
    
    @applyButton.disable
    
    closeWindow = FXButton.new(
                buttons, 
                "&Close", 
                nil, 
                self, 
                ID_ACCEPT,
                FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
    
    closeWindow.setDefault  
    closeWindow.setFocus
    
  end
  
  def populateTable
    @eventsTable.appendRows(@theEventList.size)
    
    rowIndex = 0
    
    
    @theEventList.each {
        |event|        

        eventType = EventTableItem.new(event.eventType, event)
        eventType.justify = FXTableItem::CENTER_X

        @eventsTable.setItem(
            rowIndex, 
            0,
            eventType)

        eventDescription = EventTableItem.new(event.eventDescription, event)
        eventDescription.justify = FXTableItem::CENTER_X
        
        @eventsTable.setItem(
            rowIndex, 
            1,
            eventDescription)

        timeString = formatTime(Time.at(event.eventTime))
        eventTime = EventTableItem.new(timeString, event)  
        eventTime.justify = FXTableItem::CENTER_X
        
        @eventsTable.setItem(
            rowIndex, 
            2,
            eventTime)
        
        rowIndex += 1
    }    
  end

  def cellDeselected(source, selector, event)
      if event.col == 2 then
        begin
            Time.parse(@eventsTable.getItem(event.row, 2).text)             
        rescue
            FXMessageBox::error(
                  self,
                  MBOX_OK,
                  "Can't Parse Event Time",
                  "Dates must be in mm/dd/yyyy hh:mm:ss format")
        end    
    end

    @applyButton.enable
 end
  
  def applyChanges(source, selector, event)
    #get rid of any deleted items

    @applyButton.disable
    
    @eventsToDelete.each {
        |item|
        @theEventList.delete(item)
    }
    
    0.upto(@theEventList.size - 1) {
        |index|
    
        eventData = @theEventList[index]

        eventData.eventType = @eventsTable.getItem(index, 0).text

        eventData.eventDescription = @eventsTable.getItem(index, 1).text
                    
        begin  
            newDate = Time.parse(@eventsTable.getItem(index, 2).text)
                   
            eventData.eventTime = 
                Time.local(
                    newDate.year, 
                    newDate.month, 
                    newDate.day, 
                    newDate.hour, 
                    newDate.min, 
                    newDate.sec).to_i
        rescue
            # do nothing
        end    
          
      }
  end
      
  def deleteRow(source, selector, event)
      
      if @eventsTable.selStartRow >= 0 then
        @applyButton.enable
        
        @eventsTable.selStartRow.upto(@eventsTable.selEndRow) {
            |row|
            @eventsToDelete.push(@eventsTable.getItem(row, 0).theEvent)
        }
        
        @eventsTable.removeRows(@eventsTable.selStartRow, @eventsTable.selEndRow - @eventsTable.selStartRow + 1)  
      end
  end
  
   def showTableContextMenu(source, selector, event)
      if @eventsTable.anythingSelected? then
          popupMenu = FXMenuPane.new(source)
          
          FXMenuCommand.new(popupMenu, "Delete Row(s)").connect(SEL_COMMAND,method(:deleteRow))      
          
          popupMenu.create
          
          popupMenu.popup(nil, event.root_x, event.root_y)
      end
   end

   def formatTime(currTime)
      retString = 
         sprintf(
            "%0.2d/%0.2d/#{currTime.year} %0.2d:%0.2d:%0.2d",
            currTime.month,
            currTime.day,
            currTime.hour,
            currTime.min,
            currTime.sec)
            
      return retString
   end

end

end